// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file StandardProblemIteration.h */

#ifndef AMDIS_STANDARDPROBLEMITERATION_H
#define AMDIS_STANDARDPROBLEMITERATION_H

#include "AMDiS_fwd.h"
#include "ProblemIterationInterface.h"

namespace AMDiS {

  /// A master problem for a single non coupled problem.
  class StandardProblemIteration : public ProblemIterationInterface
  {
  public:
    /// constructor
    StandardProblemIteration(ProblemStatBase *prob)
      : problem(prob)
    {}

    virtual ~StandardProblemIteration() {}

    /// Implementation of \ref ProblemIterationIterface::beginIteration()
    virtual void beginIteration(AdaptInfo *adaptInfo);

    /// Implementation of \ref ProblemIterationInterface::oneIteration()
    virtual Flag oneIteration(AdaptInfo *adaptInfo, Flag toDo);

    /// Implementation of \ref ProblemIterationInterface::endIteration()
    virtual void endIteration(AdaptInfo *adaptInfo);

    /// Implementation of \ref ProblemIterationInterface::getNumProblems()
    int getNumProblems() 
    { 
      return 1; 
    }

    /// Implementation of \ref ProblemIterationInterface::getProblem(int)
    ProblemStatBase *getProblem(int number = 0) 
    {
      return problem;
    }

    /// Returns the name of the problem.
    virtual std::string getName();

    /// Function that serializes the problem plus information about the iteration.
    virtual void serialize(std::ostream &out);

    /// Function that deserializes the problem plus information about the iteration.
    virtual void deserialize(std::istream &in);

    /// Nested assemblage and mesh adaption.
    Flag buildAndAdapt(AdaptInfo *adaptInfo, Flag toDo);

  protected:
    /// The problem to solve.
    ProblemStatBase *problem;

    /// Info level
    static int info;
  };

}

#endif

